// stores/room.ts - 管理房间状态
import { defineStore } from 'pinia'
import { ref, reactive, computed } from 'vue'
import { roomAPI, type Room, type RoomUser, type RecentRoom } from '@/services/api/room'
import { useUserStore } from '@/stores/user'
import Taro from '@tarojs/taro'

// 重新导出类型以便其他组件使用
export type { Room, RoomUser, RecentRoom } from '@/services/api/room'

export const useRoomStore = defineStore('room', () => {
  // 获取userStore
  const userStore = useUserStore()
  
  // 当前房间信息
  const currentRoom = ref<Room | null>(null)
  
  // 最近加入的房间
  const recentRooms = ref<RecentRoom[]>([])
  
  // 历史记录状态
  const historyState = reactive({
    activeRooms: [] as Room[],
    endedGames: [] as any[],
    isLoading: false
  })
  
  // 设置当前房间
  function setCurrentRoom(room: Room | null) {
    currentRoom.value = room
  }
  
  // 加载最近房间列表 - 使用统一的数据源
  async function loadRecentRooms() {
    // 首先确保活跃房间已加载 
    if (historyState.activeRooms.length === 0) {
      await loadActiveRooms()
    }
    
    // 从已加载的活跃房间和历史记录构建最近房间列表
    const recentList: RecentRoom[] = [
      // 活跃房间
      ...historyState.activeRooms.map(room => ({
        id: room.id,
        name: room.name,
        game: room.gameTitle,
        time: room.createTime,
        needPassword: room.visibility === 'private',
        status: room.status
      })),
      // 已结束游戏
      ...historyState.endedGames.filter((game, index) => index < 5).map(game => ({
        id: game.roomId || game.id,
        name: game.roomName,
        game: game.gameTitle,
        time: game.endTime,
        status: 'ended'
      }))
    ]
    
    // 按时间排序并只保留最近的条目
    recentRooms.value = recentList
      .sort((a, b) => b.time - a.time)
      .slice(0, 10)
    
    return recentRooms.value
  }
  
  // 加载活跃房间 - 统一数据源
  async function loadActiveRooms() {
    try {
      historyState.activeRooms = await roomAPI.getActiveRooms(userStore.openid)
      return historyState.activeRooms
    } catch (error) {
      console.error('加载活跃房间失败:', error)
      return []
    }
  }
  
  // 加载已结束游戏 - 统一数据源
  async function loadEndedGames() {
    try {
      historyState.endedGames = await roomAPI.getEndedGames(userStore.openid)
      return historyState.endedGames
    } catch (error) {
      console.error('加载已结束游戏失败:', error)
      return []
    }
  }
  
  // 检查用户是否已注册，如果未注册则返回错误
  async function checkUserRegistered(): Promise<{valid: boolean, message?: string}> {
    if (!userStore.isRegistered) {
      return { 
        valid: false, 
        message: '需要先完善个人信息才能继续操作' 
      }
    }
    return { valid: true }
  }
  
  // 创建房间 - 调用API并更新store
  async function createRoom(roomData: Room) {
    try {
      // 检查用户是否已注册
      const userCheck = await checkUserRegistered()
      if (!userCheck.valid) {
        return { roomId: null, success: false, message: userCheck.message }
      }
      
      const result = await roomAPI.createRoom(roomData, {
        openid: userStore.openid,
        nickname: userStore.nickname,
        avatar: userStore.avatar
      })
      
      if (result.success && result.roomId) {
        // 获取完整房间信息
        const room = await roomAPI.getRoomInfo(result.roomId)
        if (room) {
          setCurrentRoom(room)
          
          // 更新最近房间列表
          await loadRecentRooms()
        }
      }
      
      return result
    } catch (error) {
      console.error('创建房间失败:', error)
      return { roomId: null, success: false, message: '创建房间失败' }
    }
  }
  
  // 加入房间 - 调用API并更新store
  async function joinRoom(roomId: string) {
    try {
      // 检查用户是否已注册
      const userCheck = await checkUserRegistered()
      if (!userCheck.valid) {
        return { success: false, message: userCheck.message }
      }
      
      // 构建用户数据
      const userData: RoomUser = {
        id: userStore.openid,
        name: userStore.nickname,
        avatar: userStore.avatar,
        role: 'player',
        joinTime: Date.now()
      }
      
      // 调用API加入房间
      const result = await roomAPI.joinRoom(roomId, userData)
      
      if (result.success && result.roomData) {
        // 更新当前房间
        setCurrentRoom(result.roomData)
        
        // 更新用户当前房间ID
        userStore.setCurrentRoom(roomId)
        
        // 重新加载活跃房间和最近房间列表以保持数据一致性
        await loadActiveRooms()
        await loadRecentRooms()
      }
      
      return result
    } catch (error) {
      console.error('加入房间失败:', error)
      return { success: false, message: '加入房间失败' }
    }
  }
  
  // 退出房间 - 调用API并更新store
  async function leaveRoom(roomId: string) {
    try {
      const result = await roomAPI.leaveRoom(roomId, userStore.openid)
      
      if (result.success) {
        // 清除当前房间信息
        clearRoom()
        
        // 清除用户当前房间ID
        userStore.setCurrentRoom('')
      }
      
      return result
    } catch (error) {
      console.error('退出房间失败:', error)
      return { success: false, message: '退出房间失败' }
    }
  }
  
  // 加载房间信息 - 通过API获取
  async function loadRoomInfo(roomId: string) {
    try {
      const room = await roomAPI.getRoomInfo(roomId)
      
      if (room) {
        setCurrentRoom(room)
        return { success: true, room }
      }
      
      return { success: false, message: '房间不存在' }
    } catch (error) {
      console.error('加载房间信息失败:', error)
      return { success: false, message: '加载房间信息失败' }
    }
  }
  
  // 添加用户到当前房间
  function addUserToRoom(user: RoomUser) {
    if (currentRoom.value) {
      // 检查是否已存在
      const existingUserIndex = currentRoom.value.users.findIndex(u => u.id === user.id)
      if (existingUserIndex !== -1) {
        // 更新现有用户信息
        currentRoom.value.users[existingUserIndex] = {
          ...currentRoom.value.users[existingUserIndex],
          ...user
        }
      } else {
        // 添加新用户
        currentRoom.value.users.push(user)
      }
    }
  }
  
  // 从当前房间移除用户
  function removeUserFromRoom(userId: string) {
    if (currentRoom.value) {
      currentRoom.value.users = currentRoom.value.users.filter(u => u.id !== userId)
    }
  }
  
  // 更新用户信息
  function updateUserInRoom(userId: string, data: Partial<RoomUser>) {
    if (currentRoom.value) {
      const userIndex = currentRoom.value.users.findIndex(u => u.id === userId)
      if (userIndex !== -1) {
        currentRoom.value.users[userIndex] = {
          ...currentRoom.value.users[userIndex],
          ...data
        }
      }
    }
  }
  
  // 更新房间状态
  function updateRoomStatus(status: 'waiting' | 'playing' | 'ended') {
    if (currentRoom.value) {
      currentRoom.value.status = status
    }
  }
  
  // 清除当前房间
  function clearRoom() {
    currentRoom.value = null
  }
  
  // 辅助方法：获取当前房间用户数量
  function getUserCount() {
    return currentRoom.value?.users.length || 0
  }
  
  // 辅助方法：检查房间是否已满
  function isRoomFull() {
    if (!currentRoom.value) return false
    return currentRoom.value.users.length >= currentRoom.value.maxPlayers
  }
  
  // 辅助方法：获取主持人信息
  function getHoster() {
    if (!currentRoom.value) return null
    return currentRoom.value.users.find(u => u.role === 'hoster') || null
  }
  
  // 辅助方法：获取玩家列表（不包括主持人）
  function getPlayers() {
    if (!currentRoom.value) return []
    return currentRoom.value.users.filter(u => u.role === 'player')
  }
  
  // 检查用户是否在房间中
  function isUserInRoom(userId: string) {
    if (!currentRoom.value) return false
    return currentRoom.value.users.some(u => u.id === userId)
  }
  
  // 获取用户在房间中的角色
  function getUserRole(userId: string): 'hoster' | 'player' | null {
    if (!currentRoom.value) return null
    const user = currentRoom.value.users.find(u => u.id === userId)
    return user ? user.role : null
  }
  
  // 更新房间设置
  function updateRoomSettings(settings: Partial<Room>) {
    if (currentRoom.value) {
      currentRoom.value = {
        ...currentRoom.value,
        ...settings
      }
    }
  }
  
  // 加载历史记录 - 修改为使用共享数据源
  async function loadHistoryByTab(tabIndex: number) {
    historyState.isLoading = true
    
    try {
      if (tabIndex === 0) { // 最近游戏
        // 加载进行中的房间和已结束的游戏
        await loadActiveRooms()
        await loadEndedGames()
        
      } else if (tabIndex === 1) { // 我创建的
        // 确保数据已加载
        await loadActiveRooms()
        await loadEndedGames()
        
        // 筛选我创建的内容
        historyState.activeRooms = historyState.activeRooms.filter(
          room => room.hosterId === userStore.openid
        )
        historyState.endedGames = historyState.endedGames.filter(
          game => game.hosterId === userStore.openid
        )
        
      } else if (tabIndex === 2) { // 我参与的
        // 确保数据已加载
        await loadActiveRooms()
        await loadEndedGames()
        
        // 筛选我参与但不是我创建的内容
        historyState.activeRooms = historyState.activeRooms.filter(
          room => room.hosterId !== userStore.openid && 
                room.users && 
                room.users.some(u => u.id === userStore.openid)
        )
        historyState.endedGames = historyState.endedGames.filter(
          game => game.hosterId !== userStore.openid &&
                game.users && 
                game.users.some(u => u.id === userStore.openid)
        )
      }
      
      // 每次加载历史记录后更新最近房间列表
      await loadRecentRooms()
    } catch (error) {
      console.error('加载历史记录失败:', error)
    } finally {
      historyState.isLoading = false
    }
  }
  
  // 为join页面提供的限制条数的计算属性
  const limitedRecentRooms = computed(() => {
    return recentRooms.value.slice(0, 2) // 只显示最近3条
  })
  
  // 修改 joinRoomById 方法，确保返回房间数据
  async function joinRoomById(roomId: string, password?: string): Promise<{
    success: boolean, 
    message?: string, 
    needPassword?: boolean,
    roomData?: Room  // 添加房间数据
  }> {
    try {
      // 1. 检查用户是否注册
      const userCheck = await checkUserRegistered()
      if (!userCheck.valid) {
        return { success: false, message: userCheck.message }
      }
      
      // 2. 验证房间
      const checkResult = await roomAPI.checkRoomExist(roomId)
      if (!checkResult || !checkResult.exists) {
        return { success: false, message: '房间不存在' }
      }
      
      // 3. 验证密码
      if (checkResult.needPassword && !password) {
        return { success: false, message: '需要密码', needPassword: true }
      }
      
      if (checkResult.needPassword && password) {
        const pwdResult = await roomAPI.verifyRoomPassword(roomId, password)
        if (!pwdResult || !pwdResult.valid) {
          return { success: false, message: '房间密码错误' }
        }
      }
      
      // 4. 加入房间
      const joinResult = await joinRoom(roomId)
      
      // 返回结果时同时包含房间数据
      return joinResult
      
    } catch (error) {
      console.error('加入房间失败:', error)
      return { success: false, message: '加入房间失败' }
    }
  }
  
  // 添加一个根据房间状态执行跳转的辅助方法
  function navigateToRoomPage(roomId: string) {
    // 获取当前房间状态
    const status = currentRoom.value?.status
    
    if (status === 'playing') {
      // 如果房间状态为"进行中"，跳转到游戏页面
      Taro.navigateTo({
        url: `/pages/room/play/index?roomId=${roomId}`
      })
    } else {
      // 如果房间状态为"等待中"或其他状态，跳转到等待页面
      Taro.navigateTo({
        url: `/pages/room/waiting/index?roomId=${roomId}`
      })
    }
  }
  
  // 返回状态和方法
  return {
    // 状态
    currentRoom,
    recentRooms,
    limitedRecentRooms, // 新增计算属性
    historyState,
    
    // 房间操作方法
    setCurrentRoom,
    createRoom,
    joinRoom,
    leaveRoom,
    loadRoomInfo,
    loadRecentRooms,
    loadActiveRooms,
    loadEndedGames,
    clearRoom,
    
    // 用户管理方法
    addUserToRoom,
    removeUserFromRoom,
    updateUserInRoom,
    isUserInRoom,
    getUserRole,
    
    // 房间属性方法
    updateRoomStatus,
    updateRoomSettings,
    
    // 辅助方法
    getUserCount,
    isRoomFull,
    getHoster,
    getPlayers,
    
    // 历史记录方法
    loadHistoryByTab,
    joinRoomById, // 添加新方法
    navigateToRoomPage, // 添加新方法
  }
})